#include "fosfft.h"
#include <stdlib.h>
#include <stdio.h>
#include <math.h>

#define MAX_N (1<<20)
#define MAX_VARIANCE (1e-12)

int main(int argc, char **argv) {
  int retval = 0;
  int n, i;
  double deviation, variance;
  fosfft_cplx_t original[MAX_N];
  fosfft_cplx_t frequency[MAX_N];
  fosfft_cplx_t restored[MAX_N];
  fosfft_t *forward = fosfft_init_forward(MAX_N);
  fosfft_t *reverse = fosfft_init_reverse(MAX_N);
  srandomdev();
  for (n=1; n<=MAX_N; n*=2) {
    for (i=0; i<n; i++)
      original[i] =
      (random() + I*random()) / (fosfft_real_t)((1<<31)-1) * 2 - 1;
    fosfft(forward, original, frequency, n);
    fosfft(reverse, frequency, restored, n);
    variance = 0.0;
    for (i=0; i<n; i++) {
      deviation = (original[i] - restored[i] / n);
      variance += deviation * deviation;
    }
    variance /= n;
    printf("Average variance for n=%i: %e ", i, variance);
    if (variance >= 1e-12) {
      puts("FAIL");
      retval = 1;
    } else {
      puts("OK");
    }
    fflush(stdout);
  }
  return retval;
}
