#pragma once

#include <stdbool.h>
#include <complex.h>

/*
 *  floating point type used by library
 */
typedef float fosfft_real_t;
typedef complex float fosfft_cplx_t;

/*
 *  opaque data structure holding pre-calculated values for executing FFTs
 *  later
 */
typedef struct fosfft_struct fosfft_t;

/*  pre-calculate values for executing FFTs (or IFFTs) later; returned
 *  structure shall be free()ed by caller when not needed anymore;
 *  `max_n` _MUST_ be a power of 2 (elsewise behavior is undefined!)
 */
fosfft_t *fosfft_init(int max_n, bool reverse);
#define fosfft_init_forward(max_n) fosfft_init((max_n), false)
#define fosfft_init_reverse(max_n) fosfft_init((max_n), true)

/*  perform FFT (or IFFT, depending on initialization of `env`);
 *  `src` and `dst` must point to non-overlapping memory;
 *  factor 1/n is included neither in FFT nor in IFFT and must be
 *  applied by the caller where necessary;
 *  `n` _MUST_ be a power of 2 and equal to or less than `max_n` as
 *  passed to initialization function (elsewise behavior is undefined!)
 */
void fosfft(
  fosfft_t *env,
  const fosfft_cplx_t *restrict src,
  fosfft_cplx_t *restrict dst,
  int n
);

